#include "bullybgfile.h"
#include <assert.h>
#include <stdio.h>
#include <string>
#include "eriksul.h"
using namespace std;

void Bully::writeBGFile( const char *path, BGFile &bgFile ){
	assert(path);
	assert(bgFile.data);
	assert(bgFile.width > 0 && bgFile.height > 0);
	assert(bgFile.channels == 1 || bgFile.channels == 3);
	assert(bgFile.depth == DEPTH_8U);

	FILE *f = 0;
	bool error = false;
#ifdef ERIKSUL_VISUAL_STUDIO_2005
	errno_t err;
	err = fopen_s(&f, path, "wb");
	error = (f == 0 || err != 0);
#else
	f = fopen(path, "wb");
	error = (f == 0);
#endif

	if( error )
		throw string("Could not open file for writing: ") + string(path);

	fwrite(&(bgFile.width), 4, 1, f);
	fwrite(&(bgFile.height), 4, 1, f);
	fwrite(&(bgFile.channels), 4, 1, f);
	int d = (int)bgFile.depth;
	fwrite(&d, 4, 1, f);
	fwrite(bgFile.channelSeq, 4, 1, f);
	int imageSize = bgFile.width*bgFile.height*bgFile.channels*(bgFile.depth/8);
	fwrite(bgFile.data, imageSize, 1, f);

	fclose(f);
}
void Bully::readBGFile( const char *path, BGFile &bgFile ){
	assert(path);
	assert(bgFile.data == 0);

	bool error = false;
	FILE *f = 0;
#ifdef ERIKSUL_VISUAL_STUDIO_2005
	errno_t err;
	err = fopen_s(&f, path, "rb");
	error = (f == 0 || err != 0);
#else
	f = fopen(path, "rb");
	error = (f == 0);
#endif
	if( error )
		throw string("Could not open file for reading: ") + string(path);

	fread(&(bgFile.width), 4, 1, f);
	fread(&(bgFile.height), 4, 1, f);
	fread(&(bgFile.channels), 4, 1, f);
	int d = (int)bgFile.depth;
	fread(&d, 4, 1, f);
	fread(bgFile.channelSeq, 4, 1, f);
	bgFile.imageSize = bgFile.width*bgFile.height*bgFile.channels*(bgFile.depth/8);
	bgFile.data = new unsigned char[bgFile.imageSize];
	fread(bgFile.data, bgFile.imageSize, 1, f);

	fclose(f);
}
void Bully::freeBGFile( BGFile &bgFile ){
	delete[] bgFile.data;
	bgFile.data = 0;
}
