#include "bullybuffermanager.h"
#include "bullybuffer.h"
#include <assert.h>
#include <iostream>
#include "eriksul.h"
using namespace std;

Bully::BufferManager::BufferManager(){
	init();
}
Bully::BufferManager::~BufferManager(){
	destroy();
}
void Bully::BufferManager::init(){
}
void Bully::BufferManager::init( int w, int h ){
	init();
	createBuffer(w,h, 3, DEPTH_8U, "output");
}
void Bully::BufferManager::destroy(){
	std::vector<Buffer*>::iterator it = bufferList.begin();
	for( ; it != bufferList.end(); it++ ){
		delete *it;
		*it = 0;
	}
}
Bully::Buffer* const Bully::BufferManager::createBuffer( const char *fromBuffer, const char * dstname ){
	return createBuffer(get(fromBuffer), dstname);
}
Bully::Buffer* const Bully::BufferManager::createBuffer( Buffer *fromBuffer, const char * dstname ){
	return createBuffer(fromBuffer->w(), fromBuffer->h(), fromBuffer->channels(), fromBuffer->depth(), dstname);
}
Bully::Buffer* const Bully::BufferManager::createBuffer( int dstw, int dsth, int dstchannels, PixelDepth dstdepth, const char * dstname ){
	assert( dstname );
	if( bufferMap[dstname].b == 0 ){
		bufferList.push_back( new Buffer( dstname, dstw, dsth, dstchannels, dstdepth ) );
		bufferMap[dstname].b = bufferList.back();
	}
	assert( bufferMap[dstname].b->w() == dstw );
	assert( bufferMap[dstname].b->h() == dsth );
	assert( bufferMap[dstname].b->channels() == dstchannels );
	assert( bufferMap[dstname].b->depth() == dstdepth );
	return bufferMap[dstname].b;
}
Bully::Buffer* const Bully::BufferManager::createReference( const char * oldname, const char * newname ){
	// Make sure the old name already exists.
	assert(bufferMap[oldname].b);
	bufferMap[newname].b = bufferMap[oldname].b;
	return bufferMap[newname].b;
}
Bully::Buffer* const Bully::BufferManager::rename( const char * oldname, const char * newname ){
	createReference(oldname, newname);
	bufferMap[newname].b->name(newname);
	// Remove oldname.
	bufferMap.erase(oldname);
	return bufferMap[newname].b;
}
Bully::Buffer* const Bully::BufferManager::copy( unsigned char	*srcData, const char *destName ){
	assert(bufferMap[destName].b);
	bufferMap[destName].b->copy(srcData);
	return bufferMap[destName].b;
}
Bully::Buffer* const Bully::BufferManager::copy( const char * srcName, const char * destName ){
	assert(bufferMap[destName].b);
	assert(bufferMap[srcName].b);
	bufferMap[destName].b->copy(bufferMap[srcName].b);
	return bufferMap[destName].b;
}
void Bully::BufferManager::printInfo(){
	vector<Buffer*>::iterator it = bufferList.begin();
	for( ; it != bufferList.end(); it++ ){
		cout << (*it)->printInfo() << endl;
	}
}
