#pragma once

#include "bullytypes.h"
#include <vector>
#include <map>
#include <assert.h>

namespace Bully {

class Buffer;

class BufferManager {
public:
	BufferManager();
	virtual ~BufferManager();

	void init( int w, int h );

	Buffer* const createBuffer( const char * fromBuffer, const char * dstname );
	Buffer* const createBuffer( Buffer *fromBuffer, const char * dstname );
	Buffer* const createBuffer( int dstw, int dsth, int dstchannels, PixelDepth dstdepth, const char * dstname );
	Buffer* const createReference( const char * oldname, const char * newname );
	Buffer* const rename( const char * oldname, const char * newname );
	Buffer* const copy( unsigned char	*srcData,	const char *destName );
	Buffer* const copy( const char		*srcName,	const char *destName );

	Buffer* const get( const char * name ){
		assert(bufferMap[name].b);
		return bufferMap[name].b;
	}
	Buffer* const operator()(const char * name){ return get(name); }
	int count(){ return (int)bufferList.size(); }
	bool exists( const char *name ){ return bufferMap.count(name) > 0; }

	void printInfo();

protected:
	void init();
	void destroy();

	struct BMEntry {
		Buffer * b;
		BMEntry() : b(0){};
		bool operator()(const char* s1, const char* s2) const { return strcmp(s1, s2) < 0; }
	};
	std::map<const char *, BMEntry, BMEntry> bufferMap;
	std::vector<Buffer*> bufferList;
};

}; // namespace Bully
