#pragma once

#include <assert.h>
#include <vector>

namespace Bully {

class MatchMat {
public:
	MatchMat(int rows, int cols){
		_rows = rows;
		_cols = cols;
		m = new int[rows*cols];
		memset(m,0,rows*cols*sizeof(int));
	}
	virtual ~MatchMat(){
		delete[] m; m=0;
	}
	inline int & operator()(int row, int col){
		assert(row >= 0 && row < _rows && col >= 0 && col < _cols );
		return m[row*_cols + col];
	}
	int rows(){ return _rows; }
	int cols(){ return _cols; }
	void print( const char * name ){
		printf( "%s", name );
		printf( "[\n" );
		for( int r = 0; r < _rows; r++ ){
			for( int c = 0; c < _cols; c++ ) printf("%i ", m[r*_cols + c]);
			printf("\n");
		}
		printf( "]\n" );
	}
private:
	int *m;
	int _rows,_cols;
};

class MatchStr {
public:
	class Element : public std::vector<int> {
	public:
		bool matchedTo(int e){
			for( std::vector<int>::iterator it = begin(); it != end(); it++ )
				if( *it == e ) return true;
			return false;
		}
		int size(){
			return static_cast<int>(std::vector<int>::size()); 
		}
		bool operator== (int e){ return matchedTo(e); }
	};

	MatchStr(MatchMat &M){
		_count = M.rows();
		str = new Element[_count];

		for( int i = 0; i < M.rows(); i++ ){
			for( int j = 0; j < M.cols(); j++ ){
				if( M(i,j) == 1 )
					str[i].push_back(j);
			}
		}
	}
	virtual ~MatchStr(){
		delete[] str; str = 0;
	}
	Element & operator()(int i){
		assert(i >= 0 && i < _count);
		return str[i];
	}
	int count(){ return _count; }
	void print( const char * name ){
		printf( "%s", name );
		for( int i = 0; i < _count; i++ ){
			if( str[i].empty() ) printf("E,");
			else{
				printf( "(" );
				for( int j = 0; j < str[i].size(); j++ ){
					if( j < (int)str[i].size() - 1 )
						printf( "%i,", str[i][j] );
					else
						printf( "%i", (int)str[i][j] );
				}
				printf( ")" );
			}
		}
		printf( "\n" );
	}
private:
	int _count;
	Element *str;
};

};
