#include "bullymoduleanal_chase.h"
#include "bullyobjectlist.h"
#include "bullyblob.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullyparams.h"
#include <string>
#include <sstream>
#include <iostream>
#include <FLOAT.H>
using namespace std;

namespace Bully {

ModuleAnalyze_Chase::ModuleAnalyze_Chase(BufferManager &buffer, Params &params)
	: ModuleAnalyze(buffer,params)
{
}
ModuleAnalyze::Result ModuleAnalyze_Chase::process( ObjectList *objectList ){
	++_frameCount;
	Result res;

	int chaseFrames = (int)params->get("analChaseFrames");
	double chaseMaxDiff = params->get("analChaseDirDiffUpper");
	double chaseMinDist = params->get("analChaseDistDiffLower");
	double chaseMinSpeed = params->get("analChaseSpeedLower");

	for( int i = objectList->first(); i < objectList->count(); i++, objectList->next() ){
		WorldObjectInfo *ti = &(objectList->current().completeWorldInfo);
		Object &oi = objectList->current();

		// If this guy is not running, ignore him.
		if( ti->meanspeed() < chaseMinSpeed ) continue;

		if( (int)params->get("printObjectSpeed") != 0 )
			cout << "vel(" << ti->meanspeed() << ") ";

		objectList->pushPos();
		objectList->next();
		for( int j = i+1; j < objectList->count(); j++, objectList->next() ){
			WorldObjectInfo *tj = &(objectList->current().completeWorldInfo);
			Object &oj = objectList->current();

			// If this guy is not running, ignore him.
			if( tj->meanspeed() < chaseMinSpeed ) continue;

			// continue if there has not passed enough time.
			if( (int)ti->frameInfo.size() < chaseFrames + 1 ||
				(int)tj->frameInfo.size() < chaseFrames + 1 ) continue;

			// Check if the trajectories are similar.
			vector<WorldObjectFrameInfo>::reverse_iterator tithisFrame = ti->frameInfo.rbegin() + 1;
			vector<WorldObjectFrameInfo>::reverse_iterator tjthisFrame = tj->frameInfo.rbegin() + 1;

			double dist = (tithisFrame->worldFeetPos - tjthisFrame->worldFeetPos).abs();
			double dangle = .0;
			double dspeed = .0;
			double tidtarget = .0;
			double tjdtarget = .0;
			double mindist = DBL_MAX;
			int t = 0;

			while( t < chaseFrames &&
				tithisFrame != ti->frameInfo.rend() &&
				tjthisFrame != tj->frameInfo.rend())
			{
				Vector2d tithisP = tithisFrame->worldFeetPos.to2D();
				Vector2d tjthisP = tjthisFrame->worldFeetPos.to2D();

				WorldObjectFrameInfo & tinextFrame = *(tithisFrame - 1);
				WorldObjectFrameInfo & tjnextFrame = *(tjthisFrame - 1);
				Vector2d tinextP = tinextFrame.worldFeetPos.to2D();
				Vector2d tjnextP = tjnextFrame.worldFeetPos.to2D();

				Vector2d tidir = tinextP - tithisP;
				Vector2d tjdir = tjnextP - tjthisP;

				Vector2d tidist = tjthisP - tithisP;
				Vector2d tjdist = tithisP - tjthisP;

				tidtarget += fabs(tidir.angle() - tidist.angle());
				tjdtarget += fabs(tjdir.angle() - tjdist.angle());

				double dist = (tjthisP - tithisP).abs();
				if( dist < mindist ) mindist = dist;

				++t;
				++tithisFrame;
				++tjthisFrame;
			}
			assert( t > 0 );
			dangle /= double(t);
			dspeed /= double(t);
#if 0 // dist is not implemented
			tidtarget /= double(t)*dist;
			tjdtarget /= double(t)*dist;
#else
			tidtarget /= double(t);
			tjdtarget /= double(t);
#endif
			if( (tidtarget < chaseMaxDiff || tjdtarget < chaseMaxDiff) &&
				(mindist > chaseMinDist) )
			{
				Object *bully, *victim;
				if( tidtarget < tjdtarget ){
					bully = &oi;
					victim = &oj;
				}else{
					bully = &oj;
					victim = &oi;
				}

				Recti const &vr = victim->blob()->bbox;
				Recti const &br = bully->blob()->bbox;

				res.alarms.push_back(Alarm(vr, "chase: victim", "v", Vector2i(vr.w-16, 0), true));
				res.alarms.push_back(Alarm(br, "chase: bully", "b", Vector2i(br.w-16, 0), true));

				Recti totr = Recti(min(vr.x,br.x),min(vr.y,br.y),0,0);
				totr.w = max(vr.x+vr.w,br.x+br.w) - totr.x;
				totr.h = max(vr.y+vr.h,br.y+br.h) - totr.y;
				res.alarms.push_back(Alarm(totr, "chase", "chase", Vector2i(-16, 0), false));
			}

		}
		objectList->popPos();
	}

	drawDebugOutput(res);
	return res;
}

} // namespace Bully
