#include "bullymoduleanal_debug.h"
#include "bullyobject.h"
#include "bullyobjectlist.h"
#include "bullyblob.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullyparams.h"
#include <string>
#include <sstream>
#include <iostream>
using namespace std;

namespace Bully {

ModuleAnalyze_Debug::ModuleAnalyze_Debug(BufferManager &buffer, Params &params)
	: ModuleAnalyze(buffer,params)
{
	objects = 0;
}
ModuleAnalyze::Result ModuleAnalyze_Debug::process( ObjectList *objectList ){
	++_frameCount;
	objects = objectList;

	return Result();
}

void ModuleAnalyze_Debug::handleInput( bool down, char ascii ){
	updateInput(down, ascii);

	if( down && ascii == 'n' ){
		// show noise present after background subtraction and thresholding
		buffer->createBuffer(buffer->get("input_lum"), "noise");
		buffer->get("input_lum")->sub(buffer->get("background_lum"), buffer->get("noise"));
		buffer->get("noise")->threshold((unsigned char)params->get("prepForegroundDiffLower"));
		buffer->get("noise")->show();
	}
	if( down && ascii == 'b' ){
		// show all the buffers
		buffer->get("background_lum")->show();
		buffer->get("input_lum")->show();
		buffer->get("background_rgb")->show();
		buffer->get("input_rgb")->show();
		buffer->get("difference")->show();
		buffer->get("tdifference")->show();
		buffer->get("eroded")->show();
		buffer->get("smallblobsremoved")->show();
		buffer->get("shadow")->show();
		buffer->get("pseudo")->show();
	}
	if( down && ascii == 'l' ){
		// show segmented regions overlayed on input image
		buffer->createBuffer(buffer->get("input_rgb"), "segreg");
		buffer->get("segreg")->copy(buffer->get("input_rgb"));
		buffer->get("segreg")->copy(buffer->get("pseudo"), buffer->get("foreground"));
		buffer->get("segreg")->show();
	}
	if( down && ascii == 'h' ){
		// show homography
		Bully::Matrix3<double> H;
		params->getMatrix(params->getString("trackHomography",0) , H);
		cout << "Homography = (" << endl << H << ")" << endl;
	}
}
void ModuleAnalyze_Debug::handleInput( bool down, int mbutton, int x, int y ){
	updateInput(down, mbutton, x, y);
	// If the mouse button was released, and not pressed, then return.
	if( !down ) return;

	if( keyPressed['r'] ){
		if( mbutton == MOUSEBUTTON_LEFT )
			roi.push_back(Vector2i(x,y));
		else
			roi.clear();

		// Print roi to console
		cout << endl << "roi(";
		for( int i = 0; i < roi.size(); i++ )
			cout << roi[i];
		cout << ")" << endl;
	}
	
	if( keyPressed['p'] || keyPressed['d'] ){
		Bully::Matrix3<double> H;
		params->getMatrix(params->getString("trackHomography",0) , H);
		int width = buffer->get("input_rgb")->w();
		int height = buffer->get("input_rgb")->h();

		Vector3d imgPosH(x, height - y, 1.0);
		Vector3d floorPos = H * imgPosH;
		floorPos = floorPos / floorPos.z;

		cout	<< endl
				<< "imgPos(" << x << "," << y << ") "
				<< "floorPos" << floorPos << " ";

		if( keyPressed['d'] ){
			if( mbutton == MOUSEBUTTON_LEFT ){
				startPos = floorPos;
			}else{
				double dist = (floorPos - startPos).abs();
				cout << "dist(" << dist << ") ";
			}
		}
	}

	if( keyPressed['o'] ){
		// Check to see if any blob is clicked
		for( int i = objects->first(); i < objects->count(); i++, objects->next() ){
			Object &obj = objects->current();
			Recti const &r = obj.currentBbox();

			if( x >= r.x && x < r.x + r.w && y >= r.y && y < r.y + r.h ){
				obj.printInfo();
			}
		}
	}
}

} // namespace Bully
