#include "bullymoduleanal_notstanding_shadow.h"
#include "bullyobjectlist.h"
#include "bullyblob.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullyparams.h"
#include <string>
#include <iostream>
using namespace std;

namespace Bully {

ModuleAnalyze_NotStanding_Shadow::ModuleAnalyze_NotStanding_Shadow(BufferManager &buffer, Params &params)
	: ModuleAnalyze(buffer,params)
{
}
ModuleAnalyze::Result ModuleAnalyze_NotStanding_Shadow::process( ObjectList *objectList ){
	++_frameCount;
	assert(buffer->exists("shadow"));

	Result res;
	buffer->createBuffer(buffer->get("shadow"), "notstanding")->setZero();
	buffer->get("notstanding")->set(1, buffer->get("shadow"));

	for( int i = objectList->first(); i < objectList->count(); i++, objectList->next() ){
		Object &obj = objectList->current();
		Blob const *b = obj.blob();

		double shadowArea = double(getShadowArea(obj));
		double shadowRatio = shadowArea / double(b->area);

		if( (int)params->get("printShadowArea") != 0 )
			cout << "ShadowArea(" << shadowArea << ") ";

		// Alarm if:
		// - shadowRatio has been too small for a few frames
		// - during this period of casting a small shadow, the person has not moved significantly

		if( shadowRatio < params->get("analNotStandingShadowUpper") ){
			obj.shadowAreaRatio.push_back(shadowRatio);
			int notStandingFrames = obj.shadowAreaRatio.size();
			int analNotStandingFrames = (int)params->get("analNotStandingFrames");
			if( notStandingFrames >= analNotStandingFrames ){
				obj.completeTrajectory.calcStats(notStandingFrames);
				double meanSpeed = obj.completeTrajectory.meanspeed();
				double analNotStandingSpeedUpper = params->get("analNotStandingSpeedUpper");
				if( meanSpeed < analNotStandingSpeedUpper ){
					// This object has had a small shadow for a few frames, and has not moved.
					// Ergo: it is lying on the ground.
					res.alarms.push_back(Alarm(obj.blob()->bbox, "NotStanding", "ns"));
				}
			}
		}else{
			// Reset the list of shadow ratios when a large shadow is cast.
			obj.shadowAreaRatio.clear();
		}
	}

	drawDebugOutput(res);
	return res;
}
void ModuleAnalyze_NotStanding_Shadow::handleInput( bool down, char ascii ){
	if( ascii == 'a' ){
		buffer->get("shadow")->show();
		buffer->get("notstanding")->show();
		buffer->get("pseudo")->show();
		buffer->get("outercontour")->show();
	}
}
unsigned int ModuleAnalyze_NotStanding_Shadow::getShadowArea( Object &obj ){
	buffer->get("tmp")->copy(buffer->get("shadow"));
	Buffer *shadow = buffer->get("tmp");
	Buffer *shadow2 = buffer->get("notstanding");

	// Walk around the contour looking for a shadow pixel.
	int shadowArea = 0;
	Blob const *b = obj.blob();

	for( int i = 0; i < b->contour.pixels.size(); i++ ){
		Vector2i const &p = b->contour.pixels[i];

		if(shadow->pix8u(p.x,p.y)){
			shadowArea += shadow->floodFill(p.x, p.y, 0);
		}
	}
	return shadowArea;
}

} // namespace Bully
