#include "bullymoduleanal_tdt4725.h"
#include "bullytypes.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullyobjectlist.h"
#include "bullyblob.h"
#include <vector>
#include <profiling.h>
#include "eriksul.h"
using namespace std;

namespace Bully {

ModuleAnalyze_Tdt4725::ModuleAnalyze_Tdt4725( BufferManager &buffer, Params &params )
	: ModuleAnalyze(buffer, params)
{
}

ModuleAnalyze::Result ModuleAnalyze_Tdt4725::process( ObjectList *objectList ){
	PROFILE_BEGIN_FUNC;

	Result res;
	int txtOutput = (int)params->get("analTxtOutput");
	int chaseFrames = (int)params->get("analChaseFrames");
	double chaseMaxDiff = params->get("analChaseDirDiffUpper");
	double chaseMinDist = params->get("analChaseDistDiffLower");
	double chaseMinSpeed = params->get("analChaseSpeedLower");

	for( int i = objectList->first(); i < objectList->count(); i++, objectList->next() ){
		Trajectory *ti = &(objectList->current().completeTrajectory);
		Object &oi = objectList->current();

		// If this guy is not running, ignore him.
		if( ti->meanspeed() < chaseMinSpeed ) continue;
		if( txtOutput > 0 )
			printf("run(%.2f) ", ti->meanspeed());

		objectList->pushPos();
		for( int j = i+1; j < objectList->count(); j++ ){
			objectList->next();

			Trajectory *tj = &(objectList->current().completeTrajectory);
			Object &oj = objectList->current();

			// If this guy is not running, ignore him.
			if( tj->meanspeed() < chaseMinSpeed ) continue;

			// continue if there has not passed enough time.
			if( (int)ti->t.size() < chaseFrames + 1
				|| (int)tj->t.size() < chaseFrames + 1 )
				continue;

			// Check if the trajectories are similar.
			vector<Vector2i>::reverse_iterator tithisP = ti->t.rbegin() + 1;
			vector<Vector2i>::reverse_iterator tjthisP = tj->t.rbegin() + 1;

			double dist = (*tithisP - *tjthisP).abs();
			double dangle = .0;
			double dspeed = .0;
			double tidtarget = .0;
			double tjdtarget = .0;
			double mindist = 9999999.0;
			int t = 0;

			while( t < chaseFrames
				&& tithisP != ti->t.rend()
				&& tjthisP != tj->t.rend())
			{
				Vector2i & tinextP = *(tithisP - 1);
				Vector2i & tjnextP = *(tjthisP - 1);

				Vector2d tidir = (tinextP - *tithisP).toDouble();
				Vector2d tjdir = (tjnextP - *tjthisP).toDouble();

				Vector2d tidist = (*tjthisP - *tithisP).toDouble();
				Vector2d tjdist = (*tithisP - *tjthisP).toDouble();

				tidtarget += fabs(tidir.angle() - tidist.angle());
				tjdtarget += fabs(tjdir.angle() - tjdist.angle());

				double dist = (*tjthisP - *tithisP).abs();
				if( dist < mindist ) mindist = dist;

				++t;
				++tithisP;
				++tjthisP;
			}
			assert( t > 0 );
			dangle /= double(t);
			dspeed /= double(t);
//			tidtarget /= double(t)*dist;
//			tjdtarget /= double(t)*dist;
			tidtarget /= double(t);
			tjdtarget /= double(t);

			if( (tidtarget < chaseMaxDiff || tjdtarget < chaseMaxDiff)
				&& mindist > chaseMinDist )
			{
				Object *bully, *victim;
				if( tidtarget < tjdtarget ){
					bully = &oi;
					victim = &oj;
				}else{
					bully = &oj;
					victim = &oi;
				}

				Recti const &vr = victim->blob()->bbox;
				Recti const &br = bully->blob()->bbox;

				res.alarms.push_back(Alarm(vr, "chase: victim", "v", Vector2i(vr.w-16, 0), true));
				res.alarms.push_back(Alarm(br, "chase: bully", "b", Vector2i(br.w-16, 0), true));

				Recti totr = Recti(min(vr.x,br.x),min(vr.y,br.y),0,0);
				totr.w = max(vr.x+vr.w,br.x+br.w) - totr.x;
				totr.h = max(vr.y+vr.h,br.y+br.h) - totr.y;
				res.alarms.push_back(Alarm(totr, "chase", "chase", Vector2i(-16, 0), false));
			}

			if( txtOutput > 0 )
				printf("diff(%.2f %.2f) ", tidtarget, tjdtarget);
		}
		objectList->popPos();
	}

	if( txtOutput > 0 )
		printf("\n");

	drawDebugOutput(res);

	PROFILE_END_FUNC;
	return res;
}

} // namespace Bully
