#include "bullymodulepost_default.h"
#include "bullybuffermanager.h"
#include "bullybuffer.h"
#include "bullyparams.h"
#include "bullyobjectlist.h"
#include "bullyblob.h"
#include <vector>
#include <FLOAT.h>
using namespace std;

namespace Bully {

ModulePostprocess_Default::ModulePostprocess_Default(BufferManager &buffer, Params &params)
: ModulePostprocess(buffer,params) {
}
ModulePostprocess_Default::~ModulePostprocess_Default(){
}
void ModulePostprocess_Default::process( ObjectList *objects ){
	++_frameCount;

	int chaseFrames = (int)params->get("analChaseFrames");

	for( int i = objects->first(); i < objects->count(); i++, objects->next() ){
		Object &o = objects->current();

		// Traverse history to assemble the full trajectory required for analysis
		assembleTrajectory(&o);
		o.completeTrajectory.calcStats(chaseFrames);

		// Use homography to calculate floor plane position
		calcWorldInfo(&o);

		// Traverse history to assemble the full trajectory required for analysis
		assembleWorldTrajectory(&o);
		o.completeWorldInfo.calcStats(chaseFrames);

		if( (int)this->params->get("drawTrajectory") == 1 )
			o.completeTrajectory.draw(buffer->get("output"));
	}
}
inline void swapPoints(int &p0, int &p1){
	int tmp = p0;
	p0 = p1;
	p1 = tmp;
}
Vector2d ModulePostprocess_Default::calcImageFeetPos( Object *obj, Vector2d &vp ){
	Vector2i vpi = vp.toInt();
	Buffer *output = buffer->get("output");
	Buffer *labeled = buffer->get("labeled");
	unsigned char label = obj->id();

	// If the vertical vanishing point lies within the blob, then the feet position is the vp.
	if( labeled->inside(vpi.x, vpi.y) && labeled->pix8u(vpi.x, vpi.y) == label ) return vp;

	// Else a search is necessary
	Vector2d c = obj->centroid().toDouble();
	Vector2d imageFeetPos = c;

	// Go around the contour of the object, and find the point with the shortest distance to the vp.
	double mindist = DBL_MAX;
	vector<Vector2i> const &contour = obj->blob()->contour.pixels;
	vector<Vector2i>::const_iterator it = contour.begin();
	for( ; it != contour.end(); it++ ){
		double dist = (*it - vpi).abs();
		if( dist < mindist ){
			mindist = dist;
			imageFeetPos = (*it).toDouble();
		}
	}

	// Now project the contour point onto the line through the centroid and the vp.
	Vector2d c_vp(vp.x - c.x, vp.y - c.y);
	Vector2d c_m(imageFeetPos.x - c.x, imageFeetPos.y - c.y);
	double dot = c_vp.x * c_m.x + c_vp.y * c_m.y;
	double l = c_vp.abs();
	Vector2d proj = c_vp * dot/(l*l);

	imageFeetPos = c + proj;

#if 0
	// Follow the line starting at the centroid and ending at the vertical vp (or to the edge of the bbox/screen).
	// The position of the feet is the last point on the line which belongs to the foreground blob.

	Buffer *output = buffer->get("output");
	Buffer *labeled = buffer->get("labeled");
	unsigned char label = obj->id();
	Recti const &rect = obj->currentBbox();

	// Start at centroid
	Vector2i p0 = obj->centroid();
	// End at vertical vanishing point
	Vector2i p1 = vp.toInt();

	// Bresenham's line rasterization algorithm:
	bool steep = abs(p1.y - p0.y) > abs(p1.x - p0.x);
	if( steep ){
		swap(p0.x, p0.y);
		swap(p1.x, p1.y);
	}
	bool swappedPoints = false;
	if( p0.x > p1.x ){
		swappedPoints = true;
		swap(p0.x,p1.x);
		swap(p0.y,p1.y);
	}
	int deltax = p1.x - p0.x;
	int deltay = abs(p1.y - p0.y);
	int error = 0;
	int ystep;
	if( p0.y < p1.y )
		ystep = 1;
	else
		ystep = -1;
	int y = p0.y;
	for( int x = p0.x; x < p1.x; x++ ){
		if( steep ){
//			if( !rect.inside(y,x) ) break;
			if( labeled->pix8u(y,x) == label ){
				imageFeetPos.x = y;
				imageFeetPos.y = x;
				if( swappedPoints ) break; // If going from the vertical vanishing point, break now
			}
			output->setPixel(y,x,0,0,255);
		}else{
//			if( !rect.inside(x,y) ) break;
			if( labeled->pix8u(y,x) == label ){
				imageFeetPos.x = x;
				imageFeetPos.y = y;
				if( swappedPoints ) break; // If going from the vertical vanishing point, break now
			}
			output->setPixel(x,y,0,0,255);
		}
		error += deltay;
		if( 2*error >= deltax ){
			y = y + ystep;
			error = error - deltax;
		}
	}
#endif
	return imageFeetPos;
}
Vector2d ModulePostprocess_Default::calcImageHeadPos( Object *obj, Vector2d &vp ){
	Vector2i vpi = vp.toInt();
	Vector2d imageHeadPos;

	// Go around the contour of the object, and find the point with the largest distance to the vp.
	double maxdist = 0;
	vector<Vector2i> const &contour = obj->blob()->contour.pixels;
	vector<Vector2i>::const_iterator it = contour.begin();
	for( ; it != contour.end(); it++ ){
		double dist = (*it - vpi).abs();
		if( dist > maxdist ){
			maxdist = dist;
			imageHeadPos = (*it).toDouble();
		}
	}

	return imageHeadPos;
}
Vector3d ModulePostprocess_Default::calcWorldFeetPos( Object *obj, Matrix3d &H, Vector2d &imageFeetPos ){
	int width = buffer->get("input_rgb")->w();
	int height = buffer->get("input_rgb")->h();

	Vector3d imageFeetPosH = Vector3d(imageFeetPos.x, height - imageFeetPos.y, 1.0);
	Vector3d worldFeetPos = H * imageFeetPosH;
	worldFeetPos = worldFeetPos / worldFeetPos.z;
	return worldFeetPos;
}
void ModulePostprocess_Default::calcWorldInfo( Object *obj ){
	Matrix3<double> H;
	params->getMatrix(params->getString("trackHomography",0) , H);
	vector<Vector2d> vp;
	params->getVector(params->getString("trackVerticalVP",0) , vp);

	obj->worldInfo.frameInfo.push_back( WorldObjectFrameInfo() );
	WorldObjectFrameInfo &fi = obj->worldInfo.frameInfo.back();

	fi.frameId = _frameCount;
	fi.imageFeetPos = calcImageFeetPos(obj, vp[0]);
	fi.imageHeadPos = calcImageHeadPos(obj, vp[0]);
	fi.worldFeetPos = calcWorldFeetPos(obj, H, fi.imageFeetPos);

	if( (int)params->get("drawFeetPos") != 0 )
		buffer->get("output")->circle(fi.imageFeetPos.toInt(), 5, 255,0,0);
	if( (int)params->get("drawHeadPos") != 0 )
		buffer->get("output")->circle(fi.imageHeadPos.toInt(), 5, 0,0,255);
	if( (int)params->get("drawVPLine") != 0 ){
//		buffer->get("output")->line(vp[0].toInt(), fi.imageHeadPos.toInt(), 255,0,255);
		buffer->get("output")->line(vp[0].toInt(), obj->centroid(), 255,0,255);
	}
}
void ModulePostprocess_Default::assembleTrajectory( Object *asmObj ){
	asmObj->completeTrajectory.clear();
	std::vector<Vector2i> &t = asmObj->completeTrajectory.t;

	vector<Object*> histObjs;
	Object *thisObj = asmObj;
	while( thisObj ){
		histObjs.push_back( thisObj );
		thisObj = thisObj->beforeMerge;
	}

	int minGroupFrames = (int)this->params->get("postGroupFramesLower");

	vector<Object*>::reverse_iterator it = histObjs.rbegin();
	for( ; it != histObjs.rend(); it++ ){
		Object *o = *it;
		vector<Object*> & ohist = o->history.involvedObjects;

		if( ohist.size() == 1 ){

			Object *group = ohist[0];
			int n = (int)group->trajectory.size();

			if( it == histObjs.rbegin() || n > minGroupFrames ){
				// Use group trajectory as approx. to the individual trajectory.
				t.insert(t.end(), group->trajectory.begin(), group->trajectory.end());
			}else{
				// Interpolate between known locations.
				int in = n + 2;
				Object *o2 = *(it - 1);

				Vector2d p1 = o2->trajectory.back().toDouble();
				Vector2d p2 = o->trajectory.front().toDouble();
				Vector2d d = (p2 - p1) / in;

				if( (int)params->get("drawInterpolatedTrajectory") == 1 ){
					Vector2i tmp1 = p1.toInt();
					buffer->get("output")->rect( tmp1.x-5, tmp1.y-5, 10, 10, 0xff,0xff,0xff );
					Vector2i tmp2 = p2.toInt();
					buffer->get("output")->rect( tmp2.x-5, tmp2.y-5, 10, 10, 0xff,0xff,0xff );
				}
				for( int i = 0; i < in-1; i++ ){
					p1 = p1 + d;
					t.push_back(p1.toInt());
				}
			}
		}

		t.insert(t.end(), o->trajectory.begin(), o->trajectory.end());
	}
}
void ModulePostprocess_Default::assembleWorldTrajectory( Object *asmObj ){
	asmObj->completeWorldInfo.frameInfo.clear();
	std::vector<WorldObjectFrameInfo> &t = asmObj->completeWorldInfo.frameInfo;

	vector<Object*> histObjs;
	Object *thisObj = asmObj;
	while( thisObj ){
		histObjs.push_back( thisObj );
		thisObj = thisObj->beforeMerge;
	}

	vector<Object*>::reverse_iterator it = histObjs.rbegin();
	for( ; it != histObjs.rend(); it++ ){
		Object *o = *it;
		vector<Object*> & ohist = o->history.involvedObjects;

		if( ohist.size() == 1 ){

			Object *group = ohist[0];
			int n = (int)group->trajectory.size();

			// Use group trajectory as approx. to the individual trajectory.
			t.insert(t.end(), group->worldInfo.frameInfo.begin(), group->worldInfo.frameInfo.end());
		}

		t.insert(t.end(), o->worldInfo.frameInfo.begin(), o->worldInfo.frameInfo.end());
	}
}

} // namespace Bully
