#include "bullymodulepost_tdt4725.h"
#include "bullytypes.h"
#include "bullyobject.h"
#include "bullyobjectlist.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include <vector>
#include <profiling.h>
#include "eriksul.h"
using namespace std;

namespace Bully {

ModulePostprocess_Tdt4725::ModulePostprocess_Tdt4725( BufferManager &buffer, Params &params )
	: ModulePostprocess(buffer, params)
{
	objectList = 0;
}
ModulePostprocess_Tdt4725::~ModulePostprocess_Tdt4725(){
}

void ModulePostprocess_Tdt4725::process( ObjectList *objects ){
	PROFILE_BEGIN_FUNC;
	objectList = objects;

	int chaseFrames = (int)params->get("analChaseFrames");
	int txtOutput = (int)this->params->get("postTxtOutput");

	// Assemble the complete trajectory from the history:
	for( int i = objects->first(); i < objects->count(); i++, objects->next() ){
		assembleTrajectory(&(objects->current()));
		objects->current().completeTrajectory.calcStats(chaseFrames);

		if( (int)this->params->get("drawTrajectory") == 1 )
			objects->current().completeTrajectory.draw(buffer->get("output"));
	}

	PROFILE_END_FUNC;
}

void ModulePostprocess_Tdt4725::assembleTrajectory( Object *asmObj ){
	asmObj->completeTrajectory.clear();
	std::vector<Vector2i> &t = asmObj->completeTrajectory.t;

	vector<Object*> histObjs;
	Object *thisObj = asmObj;
	while( thisObj ){
		histObjs.push_back( thisObj );
		thisObj = thisObj->beforeMerge;
	}

	int minGroupFrames = (int)this->params->get("postGroupFramesLower");

	vector<Object*>::reverse_iterator it = histObjs.rbegin();
	for( ; it != histObjs.rend(); it++ ){
		Object *o = *it;
		vector<Object*> & ohist = o->history.involvedObjects;

		if( ohist.size() == 1 ){

			Object *group = ohist[0];
			int n = (int)group->trajectory.size();

			if( it == histObjs.rbegin() || n > minGroupFrames ){
				// Use group trajectory as approx. to the individual trajectory.
				t.insert(t.end(), group->trajectory.begin(), group->trajectory.end());
			}else{
				// Interpolate between known locations.
				int in = n + 2;
				Object *o2 = *(it - 1);

				Vector2d p1 = o2->trajectory.back().toDouble();
				Vector2d p2 = o->trajectory.front().toDouble();
				Vector2d d = (p2 - p1) / in;

				if( (int)params->get("drawInterpolatedTrajectory") == 1 ){
					Vector2i tmp1 = p1.toInt();
					buffer->get("output")->rect( tmp1.x-5, tmp1.y-5, 10, 10, 0xff,0xff,0xff );
					Vector2i tmp2 = p2.toInt();
					buffer->get("output")->rect( tmp2.x-5, tmp2.y-5, 10, 10, 0xff,0xff,0xff );
				}
				for( int i = 0; i < in-1; i++ ){
					p1 = p1 + d;
					t.push_back(p1.toInt());
				}
			}
		}

		t.insert(t.end(), o->trajectory.begin(), o->trajectory.end());
	}
}

} // namespace Bully
