#include "bullytypes.h"
#include "bullymoduleprep_cncc_hsv.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullybloblist.h"
#include <time.h>
#include <cv.h>
#include <algorithm>
#include <profiling.h>
#include "eriksul.h"
using namespace std;

#pragma intrinsic( sqrt, sin, cos, abs, fabs )

Bully::ModulePreprocess_CNCC_HSV::ModulePreprocess_CNCC_HSV( BufferManager &buffer, Params &params )
: ModulePreprocess(buffer, params) {
}
Bully::ModulePreprocess_CNCC_HSV::~ModulePreprocess_CNCC_HSV(){
}

void Bully::ModulePreprocess_CNCC_HSV::handleInput( bool down, char ascii ){
	if( ascii == 's' ){
		buffer->get("pseudo")->show();
		buffer->get("shadow")->show();
	}
}

// Input: "background" and "input": 8-bit images.
// Output: "preprocessed": binary image (0 and 255 are the only values).
void Bully::ModulePreprocess_CNCC_HSV::process(){
	PROFILE_BEGIN_FUNC;
	++_frameCount;
/*
	// Make sure the input data exists: background and current frame.
	assert(buffer->exists("background_rgb"));
	assert(buffer->exists("input_rgb"));

	// Clear results from last frame, and copy current input image to output.
	blobList->clear();
	buffer->get("output")->copy(buffer->get("input_rgb"));

	// Create buffers.
	int w = buffer->get("background_rgb")->w();
	int h = buffer->get("background_rgb")->h();
	if( !buffer->exists("background_lum") ){
		buffer->createBuffer( w, h, 1, DEPTH_8U, "background_lum" );
		buffer->get("background_rgb")->rgb2gray(buffer->get("background_lum"));
	}
	buffer->createBuffer( w, h, 1, DEPTH_8U, "input_lum" );
	buffer->get("input_rgb")->rgb2gray(buffer->get("input_lum"));

	buffer->createBuffer( w, h, 1, DEPTH_8U, "preprocessed" );
	buffer->createBuffer( w, h, 1, DEPTH_8U, "tmp" );

	buffer->get("input_lum")->sub(buffer->get("background_lum"), buffer->get("tmp"));
	buffer->get("tmp")->threshold((int)params->get("prepForegroundDiffLower"));
	buffer->get("tmp")->erode(structElement, 1, buffer->get("preprocessed"));
	removeSmallBlobs();
	buffer->get("preprocessed")->dilate(structElement, 2);
	buffer->get("preprocessed")->erode(structElement, 1);

	// Shadow removal
	buffer->createBuffer( w, h, 1, DEPTH_8U, "shadow" );
	findShadowPixels();
	buffer->get("shadow")->not_(buffer->get("tmp"));
	buffer->get("preprocessed")->and_(buffer->get("tmp"));


	removeSmallBlobs();

	labelBlobs();
	pseudoColorBlobs();

	drawDebugOutput();
	*/
	PROFILE_END_FUNC;
}
void Bully::ModulePreprocess_CNCC_HSV::findShadowPixels(){
	PROFILE_BEGIN_FUNC;
	Buffer* foreground = buffer->get("preprocessed");
	Buffer* shadow = buffer->get("shadow")->setZero();
	int width = foreground->w();
	int height = foreground->h();

	if( !buffer->exists("background_hsv") ){
		// This happens only the first iteration.
		buffer->createBuffer( width, height, 3, DEPTH_8U, "background_hsv" );
		buffer->get("background_rgb")->rgb2hsv(buffer->get("background_hsv"));
	}
	Buffer* b_hsv = buffer->get("background_hsv");
	Buffer* i_hsv = buffer->createBuffer( width, height, 3, DEPTH_8U, "input_hsv" );
	buffer->get("input_rgb")->rgb2hsv(i_hsv);

	int NW = (int)params->get("prepShadowCNCCWidth");
	assert(NW >= 7 && NW % 2 == 1);
	int N = NW/2;
	float MN = float(NW*NW);
	float divMN = 1.0f / MN;
	float shadowThreshold = (float)params->get("prepShadowCNCCHSVLower");

	for( int y = N; y < height - N; y++ ){
		for( int x = N; x < width - N; x++ ){
			if( foreground->pix8u(x,y) == 0 || (i_hsv->pix8u(x,y,2) >= b_hsv->pix8u(x,y,2)) ){
				// Not moving OR brighter than background => Cannot possibly be a shadow pixel.
				continue;
			}

			// Potential shadow pixel.
			float lumi=0.0f, lumb=0.0f, sumi=0.0f, sumb=0.0f, sumib=0.0f;
			for( int j = y - N; j < y + N; j++ ){
				for( int i = x - N; i < x + N; i++ ){
					lumi += i_hsv->pix8u(i,j,2);
					lumb += b_hsv->pix8u(i,j,2);

					sumi +=		max(0,	i_hsv->pix8u(i,j,0) * i_hsv->pix8u(i,j,0) +
										i_hsv->pix8u(i,j,1) * i_hsv->pix8u(i,j,1)) +
								i_hsv->pix8u(i,j,2) * i_hsv->pix8u(i,j,2);

					sumb +=		max(0,	b_hsv->pix8u(i,j,0) * b_hsv->pix8u(i,j,0) +
										b_hsv->pix8u(i,j,1) * b_hsv->pix8u(i,j,1)) +
								b_hsv->pix8u(i,j,2) * b_hsv->pix8u(i,j,2);

					sumib +=	max(0,	i_hsv->pix8u(i,j,0) * b_hsv->pix8u(i,j,0) +
										i_hsv->pix8u(i,j,1) * b_hsv->pix8u(i,j,1)) +
								i_hsv->pix8u(i,j,2) * b_hsv->pix8u(i,j,2);
				}
			}
			lumi *= divMN;
			lumb *= divMN;
			float vari = sumi - MN*lumi*lumi;
			float varb = sumb - MN*lumb*lumb;
			// cncc = (sumib-MN*lumi*lumb) / sqrt(vari*varb)
			float cncc = (sumib - MN*lumi*lumb) * cvInvSqrt(vari*varb);

			if( cncc >= shadowThreshold ){
				// Probably a shadow pixel.
				shadow->pix8u(x,y) = 0xFF;
			}
		}
	}
	PROFILE_END_FUNC;
}
void Bully::ModulePreprocess_CNCC_HSV::drawDebugOutput(){
	ModulePreprocess::drawDebugOutput();
}
