#include "bullytypes.h"
#include "bullymoduleprep_cncc_rgb.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullybloblist.h"
#include <time.h>
#include <cv.h>
#include <algorithm>
#include <profiling.h>
#include "eriksul.h"
using namespace std;

#pragma intrinsic( sqrt, sin, cos, abs, fabs )

Bully::ModulePreprocess_CNCC_RGB::ModulePreprocess_CNCC_RGB( BufferManager &buffer, Params &params )
: ModulePreprocess(buffer, params) {
}
Bully::ModulePreprocess_CNCC_RGB::~ModulePreprocess_CNCC_RGB(){
}

void Bully::ModulePreprocess_CNCC_RGB::handleInput( bool down, char ascii ){
	if( ascii == 's' ){
		buffer->get("pseudo")->show();
		buffer->get("shadow")->show();
	}
}

// Input: "background" and "input": 8-bit images.
// Output: "preprocessed": binary image (0 and 255 are the only values).
void Bully::ModulePreprocess_CNCC_RGB::process(){
	PROFILE_BEGIN_FUNC;
	++_frameCount;
/*
	// Make sure the input data exists: background and current frame.
	assert(buffer->exists("background_rgb"));
	assert(buffer->exists("input_rgb"));

	// Clear results from last frame, and copy current input image to output.
	blobList->clear();
	buffer->get("output")->copy(buffer->get("input_rgb"));

	// Create buffers.
	int w = buffer->get("background_rgb")->w();
	int h = buffer->get("background_rgb")->h();
	if( !buffer->exists("background_lum") ){
		buffer->createBuffer( w, h, 1, DEPTH_8U, "background_lum" );
		buffer->get("background_rgb")->rgb2gray(buffer->get("background_lum"));
	}
	buffer->createBuffer( w, h, 1, DEPTH_8U, "input_lum" );
	buffer->get("input_rgb")->rgb2gray(buffer->get("input_lum"));

	buffer->createBuffer( w, h, 1, DEPTH_8U, "preprocessed" );
	buffer->createBuffer( w, h, 1, DEPTH_8U, "tmp" );

	buffer->get("input_lum")->sub(buffer->get("background_lum"), buffer->get("tmp"));
	buffer->get("tmp")->threshold((int)params->get("prepForegroundDiffLower"));
	buffer->get("tmp")->erode(structElement, 1, buffer->get("preprocessed"));
	removeSmallBlobs();

	// Shadow removal
	buffer->createBuffer( w, h, 1, DEPTH_8U, "shadow" );
	findShadowPixels();
	buffer->get("shadow")->not_(buffer->get("tmp"));
	buffer->get("preprocessed")->and_(buffer->get("tmp"));

//	buffer->get("preprocessed")->dilate(structElement, 2);
//	buffer->get("preprocessed")->erode(structElement, 1);

	// NOTE: second removeSmallBlobs for removing fragments resulting from shadow removal.
//	removeSmallBlobs();

	labelBlobs();
	pseudoColorBlobs();

	//buildHistograms();

	drawDebugOutput();
	*/
	PROFILE_END_FUNC;
}
void Bully::ModulePreprocess_CNCC_RGB::findShadowPixels(){
	PROFILE_BEGIN_FUNC;
	Buffer* foreground = buffer->get("preprocessed");
	Buffer* shadow = buffer->get("shadow")->setZero();
	int width = foreground->w();
	int height = foreground->h();

	Buffer* b_lum = buffer->get("background_lum");
	Buffer* i_lum = buffer->get("input_lum");
	Buffer* b_rgb = buffer->get("background_rgb");
	Buffer* i_rgb = buffer->get("input_rgb");

	int N = 3;
	int ND = N*2+1;
	float MN = float(ND*ND);
	float divMN = 1.0f / MN;
	float shadowThreshold = 0.95f;

	for( int y = N; y < height - N; y++ ){
		for( int x = N; x < width - N; x++ ){
			if( foreground->pix8u(x,y) == 0 || (i_lum->pix8u(x,y) >= b_lum->pix8u(x,y)) ){
				// Not moving OR brighter than background => Cannot possibly be a shadow pixel.
				continue;
			}

			// Potential shadow pixel.
			int lumi=0, lumb=0, sumi=0, sumb=0, sumib=0;
			for( int j = y - N; j < y + N; j++ ){
				for( int i = x - N; i < x + N; i++ ){
					lumi += i_rgb->pix8u(i,j,2);
					lumb += b_rgb->pix8u(i,j,2);

					sumi +=		max(0,	i_rgb->pix8u(i,j,0) * i_rgb->pix8u(i,j,0) +
										i_rgb->pix8u(i,j,1) * i_rgb->pix8u(i,j,1)) +
								i_rgb->pix8u(i,j,2) * i_rgb->pix8u(i,j,2);

					sumb +=		max(0,	b_rgb->pix8u(i,j,0) * b_rgb->pix8u(i,j,0) +
										b_rgb->pix8u(i,j,1) * b_rgb->pix8u(i,j,1)) +
								b_rgb->pix8u(i,j,2) * b_rgb->pix8u(i,j,2);

					sumib +=	max(0,	i_rgb->pix8u(i,j,0) * b_rgb->pix8u(i,j,0) +
										i_rgb->pix8u(i,j,1) * b_rgb->pix8u(i,j,1)) +
								i_rgb->pix8u(i,j,2) * b_rgb->pix8u(i,j,2);
				}
			}
			float lumif = float(lumi) * divMN;
			float lumbf = float(lumb) * divMN;
			float sumif = float(sumi);
			float sumbf = float(sumb);
			float sumibf = float(sumib);
			float vari = sumif - MN*lumif*lumif;
			float varb = sumbf - MN*lumbf*lumbf;
			// cncc = (sumib-MN*lumi*lumb) / sqrt(vari*varb)
			float cncc = (sumibf - MN*lumif*lumbf) * cvInvSqrt(vari*varb);

			if( cncc >= shadowThreshold ){
				// Probably a shadow pixel.
				shadow->pix8u(x,y) = 0xFF;
			}
		}
	}
	PROFILE_END_FUNC;
}
void Bully::ModulePreprocess_CNCC_RGB::drawDebugOutput(){
	ModulePreprocess::drawDebugOutput();
}
