#include "bullymoduleprep_default.h"
#include "bullybuffermanager.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybloblist.h"
#include <cv.h>
#include <assert.h>
#include <iostream>

Bully::ModulePreprocess_Default::ModulePreprocess_Default(BufferManager &buffer, Params &params)
: ModulePreprocess(buffer,params) {
}
Bully::ModulePreprocess_Default::~ModulePreprocess_Default(){
}
void Bully::ModulePreprocess_Default::process(){
	++_frameCount;

	blobList->clear();

	Buffer *bg_rgb = buffer->get("background_rgb");
	Buffer *i_rgb = buffer->get("input_rgb");
	int w = bg_rgb->w();
	int h = bg_rgb->h();

	if( !buffer->exists("background_lum") ){
		buffer->createBuffer(w, h, 1, DEPTH_8U, "background_lum");
		buffer->get("background_rgb")->rgb2gray(buffer->get("background_lum"));
	}
	buffer->createBuffer(w, h, 1, DEPTH_8U, "input_lum");
	buffer->get("input_rgb")->rgb2gray(buffer->get("input_lum"));

	buffer->createBuffer(w, h, 1, DEPTH_8U, "difference");
	buffer->get("input_lum")->sub(buffer->get("background_lum"), buffer->get("difference"));
	
	buffer->createBuffer(w, h, 1, DEPTH_8U, "tdifference");
	buffer->get("difference")->threshold((int)params->get("prepForegroundDiffLower"), buffer->get("tdifference"));

	// Mask out anything not in a region of interest.
	buffer->createBuffer(w, h, 1, DEPTH_8U, "roi");
	renderROIBuffer("roi");
	buffer->get("tdifference")->and_(buffer->get("roi"));

	buffer->createBuffer(w, h, 1, DEPTH_8U, "eroded");
	buffer->get("tdifference")->erode(structElement, 1, buffer->get("eroded"));

	buffer->createBuffer(w, h, 1, DEPTH_8U, "smallblobsremoved");
	removeSmallBlobs("eroded", "smallblobsremoved");

	buffer->createBuffer(w, h, 1, DEPTH_8U, "foreground");
	buffer->get("smallblobsremoved")->dilate(structElement, 2, buffer->get("foreground"));
	buffer->get("foreground")->erode(structElement, 1, buffer->get("foreground"));

	buffer->createBuffer(w, h, 1, DEPTH_8U, "labeled");
	labelBlobs("foreground", "labeled");

	buffer->createBuffer(w, h, 3, DEPTH_8U, "pseudo");
	pseudoColorBlobs("labeled", "pseudo");

	drawDebugOutput();
}

void Bully::ModulePreprocess_Default::drawDebugOutput(){
	ModulePreprocess::drawDebugOutput();
}
