#include "bullytypes.h"
#include "bullymoduleprep_tdt4725.h"
#include "bullyparams.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullybloblist.h"
#include <time.h>
#include <cv.h>
#include <algorithm>
#include <profiling.h>
#include "eriksul.h"
using namespace std;

#pragma intrinsic( sqrt, sin, cos, abs, fabs )

Bully::ModulePreprocess_Tdt4725::ModulePreprocess_Tdt4725( BufferManager &buffer, Params &params )
	: ModulePreprocess(buffer, params)
{
}
Bully::ModulePreprocess_Tdt4725::~ModulePreprocess_Tdt4725(){
}

void Bully::ModulePreprocess_Tdt4725::handleInput( bool down, char ascii ){
	if( ascii == 's' ){
		// Show shadow pixels
		buffer->get("pseudo")->show();
	}
}

// Input: "background" and "input": 8-bit images.
// Output: "preprocessed": binary image (0 and 255 are the only values).
void Bully::ModulePreprocess_Tdt4725::process(){
	PROFILE_BEGIN_FUNC;
/*
	// Make sure the input data exists: background and current frame.
	assert(buffer->exists("background_rgb"));
	assert(buffer->exists("input_rgb"));

	// Clear results from last frame, and copy current input image to output.
	blobList->clear();
	buffer->get("output")->copy(buffer->get("input_rgb"));

	// Create buffers.
	int w = buffer->get("background_rgb")->w();
	int h = buffer->get("background_rgb")->h();
	if( !buffer->exists("background_lum") ){
		buffer->createBuffer( w, h, 1, DEPTH_8U, "background_lum" );
		buffer->get("background_rgb")->rgb2gray(buffer->get("background_lum"));
	}
	buffer->createBuffer( w, h, 1, DEPTH_8U, "input_lum" );
	buffer->get("input_rgb")->rgb2gray(buffer->get("input_lum"));

	buffer->createBuffer( w, h, 1, DEPTH_8U, "preprocessed" );
	buffer->createBuffer( w, h, 1, DEPTH_8U, "tmp" );

	buffer->get("input_lum")->sub(buffer->get("background_lum"), buffer->get("tmp"));
	buffer->get("tmp")->threshold((int)params->get("prepForegroundDiffLower"));
	buffer->get("tmp")->erode(structElement, 1, buffer->get("preprocessed"));

	removeSmallBlobs();

	buffer->get("preprocessed")->dilate(structElement, 2);
	buffer->get("preprocessed")->erode(structElement, 1);

	labelBlobs();
	pseudoColorBlobs();

	//buildHistograms();

	drawDebugOutput();
	*/
	PROFILE_END_FUNC;
}
void Bully::ModulePreprocess_Tdt4725::drawDebugOutput(){
	PROFILE_BEGIN_FUNC;
	int gfxOutput = (int)params->get("prepGfxOutput");
	for( int i = 0; i < blobList->count(); i++ ){
		Blob & b = *(blobList->get(i));

		if( 1 < gfxOutput ){
			// Draw a rectangle where the bbox is.
			buffer->get("output")->rect( b.bbox.x, b.bbox.y, b.bbox.w, b.bbox.h, 255,0,255);
		}
		if( 0 < gfxOutput ){
			// Draw an outline around the blob.
			buffer->get("output")->drawContour(b.contour.pixels, 255,0,255);
		}
	}
	PROFILE_END_FUNC;
}
