#include "bullymoduletrack_default.h"
#include "bullybuffer.h"
#include "bullybuffermanager.h"
#include "bullyparams.h"
#include "bullybloblist.h"
#include "bullyobjectlist.h"
#include "bullymatchmat.h"
#include "bullymodulepost_default.h"
#include <iostream>
using namespace std;

namespace Bully {

ModuleTrack_Default::ModuleTrack_Default(BufferManager &buffer, Params &params)
	: ModuleTrack(buffer,params)
{
	objects = new ObjectList();
	postProc = new ModulePostprocess_Default(buffer, params);
}
ModuleTrack_Default::~ModuleTrack_Default(){
	delete objects;
	delete postProc;
}
void ModuleTrack_Default::process( BlobList *blobList ){
	++_frameCount;

	// The default tracker does not attempt to handle splits and merges.

	int curN = blobList->count();
	int prevN = objects->count();

	MatchMat curM(curN,prevN);	// from current to previous
	MatchMat prevM(prevN,curN);	// from previous to current

	// Match previous to current
	for( int i = objects->first(); i < objects->count(); i++, objects->next() ){
		Object & obj = objects->current();

		for( int j = 0; j < curN; j++ ){
			Blob &blob = *(blobList->get(j));

			if( obj.predictBbox.overlaps( blob.bbox ) )
				prevM(i,j) = 1;
		}
	}

	// Match current to previous
	for( int i = 0; i < curN; i++ ){
		Blob &blob = *(blobList->get(i));

		for( int j = objects->first(); j < objects->count(); j++, objects->next() ){
			Object & obj = objects->current();

			if( obj.predictBbox.overlaps( blob.bbox ) )
				curM(i,j) = 1;
		}
	}

	// Construct the matching strings.
	MatchStr prevS(prevM);
	MatchStr curS(curM);

	// Check if any object has exited (not matched to any current blob)
	for( int i = 0; i < prevS.count(); i++ ){
		if( prevS(i).empty() ){
			objects->get(i).matchResult = Object::EXIT;
		}
	}

	// Check if any object has entered (not matched to any previous blob)
	for( int i = 0; i < curS.count(); i++ ){
		if( curS(i).empty() ){
			objects->add(Object(*(blobList->get(i))));
		}
	}

	// Check if any object is matched to *one* blob
	for( int i = 0; i < prevS.count(); i++ ){
		if( prevS(i).size() == 1 ){
			objects->get(i).update(*(blobList->get(prevS(i)[0])));
		}
	}

	// Check if *one* blob is matched to several objects
	for( int i = 0; i < curS.count(); i++ ){
		if( curS(i).size() > 1 ){
			// Mark all matched objects as exited, and introduce a new one based on the blob.
			for( int j = 0; j < curS(i).size(); j++ ){
				objects->get(curS(i)[j]).matchResult = Object::EXIT;
			}
			objects->add(Object(*(blobList->get(i))));
		}
	}

	objects->deleteExited();

	drawDebugOutput();
	printDebugOutput();

	// Call post-processing module here
	postProc->process(objects);
}
ObjectList *ModuleTrack_Default::getObjectList(){
	return objects;
}
void ModuleTrack_Default::drawDebugOutput(){
	if( (int)params->get("drawObjectRect") == 1 ){
		for( int i = objects->first(); i < objects->count(); i++, objects->next() ){
			Object &o = objects->current();
			Recti const &r = o.blob()->bbox;
			buffer->get("output")->rect( r.x, r.y, r.w, r.h, 0xFF, 0x00, 0xFF );
		}
	}
}

} // namespace Bully
