#pragma once

#include "bullymoduletrack.h"
#include "bullytypes.h"
#include <vector>
#include <map>

namespace Bully {
class ObjectList;

namespace Reading {
class RegionTracker;
class Inputs;
class Results;
}

class ModuleTrack_Reading : public ModuleTrack {
public:
	ModuleTrack_Reading( BufferManager &buffer, Params &params );
	virtual ~ModuleTrack_Reading();

	void process( BlobList * );
	void handleInput( bool down, char ascii );
	void handleInput( bool down, int mbutton, int x, int y ){};
	ObjectList * getObjectList(){ return objectList; }

private:
	void updateTrackedObjects();
	void removeOldTrackedObjects();
	void debugOutput();

	Reading::RegionTracker *readingRegionTracker;
	Reading::Inputs *inputs;
	Reading::Results *results;
	ObjectList *objectList;

	struct TrackedObject {
		TrackedObject(){
			id = 0;
			lastFrameDetected = 0;
			col[0] = col[1] = col[2] = 0;
			while( col[0] + col[1] + col[2] < 100 ){
				col[0] = rand()%256;
				col[1] = rand()%256;
				col[2] = rand()%256;
			}
		}
		int id;
		int lastFrameDetected;
		unsigned char col[3];
		Vector2i dir;
		std::vector<Vector2i> trajectory;
	};

	std::map<int, TrackedObject> trackedObjects;
};

} // namespace Bully
