#include "bullytypes.h"
#include "bullyobject.h"
#include "bullyblob.h"
#include <iostream>
#include <algorithm>
#include <assert.h>
#include "eriksul.h"
using namespace std;

//#define OBJECT_DEBUG

namespace Bully {

History::History(){
	eventType = EMPTY;
}
History::History( EventType t ){
	eventType = t;
}
History::History( const History & h ){
	eventType = h.eventType;
	involvedObjects.resize( h.involvedObjects.size() );
	copy(h.involvedObjects.begin(), h.involvedObjects.end(), involvedObjects.begin());
}
const History& Bully::History::operator=( const History & h ){
	eventType = h.eventType;
	involvedObjects.resize( h.involvedObjects.size() );
	copy(h.involvedObjects.begin(), h.involvedObjects.end(), involvedObjects.begin());
	return *this;
}
void History::incRefCount(){
	vector<Object*>::iterator it = involvedObjects.begin();
	for( ; it != involvedObjects.end(); it++ ){
		(**it).incRefCount();
	}
}
void History::decRefCount(){
	vector<Object*>::iterator it = involvedObjects.begin();
	for( ; it != involvedObjects.end(); it++ ){
		(**it).decRefCount();
	}
}

// ---------------------------------------------------------------------------
Object::Object(){
	init();
}
Object::Object( const Object & o ){
	init();
	init(o);
}
Object::Object( Blob & b ){
	init();
	Observation::init(b);

	_blob = new Blob(b);
	predictBbox = b.bbox;
	trajectory.push_back( b.centroid );
	col = b.col;
}
void Object::init(){
	Observation::init();

	_blob = 0;
	visited = false;
	_refCount = 0;
	matchResult = NONE;
	beforeMerge = 0;
}
void Object::init( const Object & o ){
	Observation::init(o);

	delete _blob;
	_blob = new Blob(*o.blob());

	imgSpeed = o.imgSpeed;
	predictBbox = o.predictBbox;
	trajectory.resize(o.trajectory.size());
	copy(o.trajectory.begin(), o.trajectory.end(), trajectory.begin());
	history = o.history;
	beforeMerge = o.beforeMerge;
	col = o.col;
}
Object::~Object(){
	delete _blob;
}
const Object& Object::operator= ( const Object & o ){
	if( &o == this ) return *this;
	init(o);
	return *this;
}

void Object::update( Blob & b ){
	// Get differences
	imgSpeed = b.centroid - centroid();

	// Copy over new values
	delete _blob;
	_blob = new Blob(b);

	trajectory.push_back( b.centroid );

	// Update predicted location: assume same dimensions, but new centroid coordinates.
	predictBbox = b.bbox;
	Vector2<int> predictCentroid = b.centroid + imgSpeed;
	predictBbox.x = predictCentroid.x - b.bbox.w/2;
	predictBbox.y = predictCentroid.y - b.bbox.h/2;
}
void Object::incRefCount(){
	++_refCount;
	history.incRefCount();
}
void Object::decRefCount(){
	--_refCount;
	history.decRefCount();
}
__int64 Object::frameLastDetected() const {
	assert(_blob);
	return _blob->frameFirstDetected;
}
__int32 Object::area() const {
	assert(_blob);
	return _blob->area;
}
Rect<int> Object::currentBbox() const {
	assert(_blob);
	return _blob->bbox;
}
Vector2<int> Object::centroid() const {
	assert(_blob);
	return _blob->centroid;
}

void Object::printInfo(){
	cout << endl
		<< "objinfo(" << endl
		<< "\tworldFeetPos=" << worldInfo.frameInfo.back().worldFeetPos << endl
		<< "\tworldMeanSpeed=" << worldInfo.meanspeed() << endl
		<< "\tcentroid=" << centroid() << endl
		<< "\tbbox=" << currentBbox() << endl
		<< "\tarea=" << area() << endl
		<< "\timgSpeed=" << imgSpeed << endl
		<< "\tshadowAreaRatio.size()=" << shadowAreaRatio.size() << " (#frames with small shadow)" << endl
		<< ")" << endl;
}

} // namespace Bully
