#pragma once

#include "bullyobservation.h"
#include "bullytypes.h"
#include "bullytrajectory.h"
#include "bullyworldobjectinfo.h"
#include <vector>
#include <list>

namespace Bully {

class Blob;
class Object;


/**
	Each object has a "history", containing a *single* event, with
	possibly many previous objects involved. Each of these have also
	a history, and the whole thing becomes a graph.
*/
class History {
public:
	std::vector<Object*> involvedObjects;
	enum EventType { EMPTY=0, MERGE, SPLIT, } eventType;

	History();
	History( EventType t );
	History( const History & h );
	const History& operator=( const History & h );
	void incRefCount();
	void decRefCount();
};

class Object : public Observation {
public:
	Object();
	Object( const Object & o );
	Object( Blob & b );
	virtual ~Object();
	const Object& operator= ( const Object & o );
	void update( Blob & b );

	Blob const * blob() const { return _blob; }
	__int64 frameLastDetected() const;
	__int32 area() const;
	Rect<int> currentBbox() const;
	Vector2<int> centroid() const;

	Rect<int> predictBbox;
	Vector2<int> imgSpeed;

	// The trajectory of this object.
	std::vector<Vector2i> trajectory;
	// The complete trajectory, including merges/splits.
	Trajectory completeTrajectory;

	// A list of cast shadow area relative to total blob area.
	std::vector<double> shadowAreaRatio;

	// Information about the 3D positions of this object
	WorldObjectInfo worldInfo;
	// Information about the 3D positions of this object, *including* merges/splits.
	WorldObjectInfo completeWorldInfo;

	// MERGE/SPLIT ---------------------------------

	History history;
	Object* beforeMerge;

	// Mainly used for drawing the trajectories.
	bool visited;
	Color col;

	void printInfo();

public:
	void incRefCount();
	void decRefCount();
	int refCount(){ return _refCount; }
private:
	int _refCount;

public:
	enum {
		NONE = 0,
		EXIT,
		UPDATE,
		SPLIT,
		MERGE,
	} matchResult;
	struct {
		int matchedToBlob;
		int mergedIntoBlob;
		std::vector<int> splitIntoBlobs;
	} matchData;

private:
	Blob * _blob;

	void init();
	void init( const Object & o );
};

} // namespace Bully

template<class T>
std::ostream& operator<<(std::ostream& os, Bully::Object const &o){
	os << "obj" << endl;
	return os;
}
