#include "bullyparams.h"
#include <fstream>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <assert.h>
using namespace std;

void Bully::Params::Param::def(const char *id, TYPE t, const char *desc){
	_id = string(id);
	_type = t;
	_desc = string(desc);
}
void Bully::Params::Param::set(double value){
	if(_type == MULTI_VALUE || _dvalue.size() == 0)
		_dvalue.push_back(value);
	else
		_dvalue[0] = value;
}
void Bully::Params::Param::set(const char *value){
	if(_type == MULTI_VALUE || _svalue.size() == 0)
		_svalue.push_back(string(value));
	else
		_svalue[0] = string(value);
}
int Bully::Params::Param::valueCount(){
	return max(_svalue.size(), _dvalue.size());
}
void Bully::Params::Param::printValue(std::ostream &s){
	if( _dvalue.size() > 0 ){
		for( int i = 0; i < _dvalue.size(); i++ )
			s << _id << " = " << _dvalue[i] << endl;
	}else if( _svalue.size() > 0 ){
		for( int i = 0; i < _svalue.size(); i++ )
			s << _id << " = " << _svalue[i] << endl;
	}
}
void Bully::Params::Param::printType(std::ostream &s){
	if( _type == SINGLE_VALUE )
		s << "SINGLE";
	else
		s << "MULTI";
}


Bully::Params::Params(){
	setDefaultValues();
}
void Bully::Params::defSingleValue(const char *id, int defaultValue, const char *desc){
	defSingleValue(id, static_cast<double>(defaultValue), desc);
}
void Bully::Params::defSingleValue(const char *id, double defaultValue, const char *desc){
	string sid = string(id);
	paramList[sid].def(id, Param::SINGLE_VALUE, desc);
	paramList[sid].set(defaultValue);
	paramSequence.push_back(&(paramList[sid]));
}
void Bully::Params::defSingleValue(const char *id, const char *defaultValue, const char *desc){
	string sid = string(id);
	paramList[sid].def(id, Param::SINGLE_VALUE, desc);
	paramList[sid].set(defaultValue);
	paramSequence.push_back(&(paramList[sid]));
}
void Bully::Params::defMultiValue(const char *id, int defaultValue, const char *desc){
	defMultiValue(id, desc);
	paramList[string(id)].set(defaultValue);
}
void Bully::Params::defMultiValue(const char *id, double defaultValue, const char *desc){
	defMultiValue(id, desc);
	paramList[string(id)].set(defaultValue);
}
void Bully::Params::defMultiValue(const char *id, const char *defaultValue, const char *desc){
	defMultiValue(id, desc);
	paramList[string(id)].set(defaultValue);
}
void Bully::Params::defMultiValue(const char *id, const char *desc){
	string sid = string(id);
	paramList[sid].def(id, Param::MULTI_VALUE, desc);
	paramSequence.push_back(&(paramList[sid]));
}
void Bully::Params::setDefaultValues(){
	// Threshold parameters are suffixed by either "Lower" or "Upper".
	// "Lower" is a lower bound, meaning the element is "in the set" if it has a higher value.
	// "Upper" is an upper bound, meaning the element is "in the set" if it has a lower value.

	// Set default values.
	defSingleValue("inputPath", "",					"path to input (video)");
	defSingleValue("backgroundPath", "",			"path to background/reference frame (output from bgtool)");

	defSingleValue("modulePreprocess",	"default", 	"preprocessing module to use");
	defSingleValue("moduleTrack", "default",		"tracking module to use");
	defMultiValue("moduleAnalyze", "default",		"analysis module to use");
	
	defSingleValue("prepForegroundDiffLower", 8, 	"preprocessing: lower bound (on difference) for a pixel to be in the foreground");
	defSingleValue("prepShadowCNCCLower", 0.80,		"preprocessing: lower bound of FG/BG NCC for a shadow pixel");
	defSingleValue("prepShadowCNCCHSVLower", 0.85, "preprocessing: lower bound (on NCC with background) for a pixel in the foreground to be shadow");
	defSingleValue("prepShadowCNCCWidth", 7, 		"preprocessing: width of (color) normalized cross-correlation (NCC) mask");
	defSingleValue("prepShadowHSVLumLower", 0.05, 	"preprocessing: lower bound on the ratio of shadow luminance to background luminance");
	defSingleValue("prepShadowHSVLumUpper", 0.98, 	"preprocessing: upper bound on the ratio of shadow luminance to background luminance");
	defSingleValue("prepShadowHSVHueUpper", 50, 	"preprocessing: upper bound on the difference in hue between shadow and background");
	defSingleValue("prepShadowHSVSatUpper", 50, 	"preprocessing: upper bound on the difference in saturation between shadow and background");
	defSingleValue("prepRegionAreaLower", 100, 		"preprocessing: lower bound on a region area for it to survive region-based noise cleaning");
	defSingleValue("prepStructElementSize", 3, 	"preprocessing: width (and height) of morphological structuring element");
	defSingleValue("prepPosition", "median", 		"preprocessing: definition of the position of a blob: \'centroid\', \'median\', \'centerbbox\'");

	defSingleValue("trackHomography", "1 0 0; 0 1 0; 0 0 1", "tracking: 3x3 homography matrix for going from the image plane to a world plane (typically the ground/floor)");
	defSingleValue("trackVerticalVP", "(0 0)", "tracking: vertical vanishing point in the image");

	defSingleValue("postGroupFramesLower", 30, 	"postprocessing: lower bound on the number of frames a group must have stayed together to have its own trajectory");

	defSingleValue("analChaseFrames", 20, 			"analysis: number of frames to analyze");
	defSingleValue("analChaseSpeedLower", 5.0/30.0, "analysis: lower bound on the speed (meters/frame) of a blob for it to be running");
	defSingleValue("analChaseDirDiffUpper", 0.5, 	"analysis: upper bound on the difference of direction for a chase to be occuring");
	defSingleValue("analChaseDistDiffLower", 1.0, 	"analysis: lower bound on the minimum distance between two blobs during a chase");
	defSingleValue("analNotStandingShadowUpper", 0.01, "analysis: upper bound on the ratio of cast shadow area to blob area for a person to be <Not Standing>");
	defSingleValue("analNotStandingFrames", 15, "analysis: number of frames to analyze");
	defSingleValue("analNotStandingSpeedUpper", 3.0, "analysis: upper bound on the mean speed (meters/frame) of a blob for it to be <Not Standing>");

	defSingleValue("drawInput", 1, 		"copy input frame to output");
	defSingleValue("drawBlobRect", 0, 		"draw a rectangle around detected blobs");
	defSingleValue("drawBlobContour", 0, 	"draw a contour around detected blobs");
	defSingleValue("drawObjectRect", 1, 		"draw a rectangle around tracked objects");
	defSingleValue("drawShadow", 0, 	"mark shadow pixels");
	defSingleValue("drawROI", 1, 		"draw a rectangle around the Region Of Interest");
	defSingleValue("drawTrajectory", 1, "draw the trajectory");
	defSingleValue("drawPredictPos", 0, "draw the predicted position of an object");
	defSingleValue("drawFeetPos", 0, "draw the estimated position of the feet of a person");
	defSingleValue("drawHeadPos", 0, "draw the estimated position of the head of a person");
	defSingleValue("drawVPLine", 0, "draw a line from the head to the vertical vanishing point");
	defSingleValue("drawInterpolatedTrajectory", 0, "draw the interpolated points of the trajectory");
	defSingleValue("drawAlarm", 1, 		"mark the area where an alarm was triggered");

	defSingleValue("printFrameCount", 0, "outputs the number of frames to the console once per frame");
	defSingleValue("printObjectCount", 0, "outputs the number of tracked objects to the console once per frame");
	defSingleValue("printObjectSpeed", 0, "outputs the speed of each tracked object to the console once per frame");
	defSingleValue("printShadowArea", 0, "outputs the area of each cast shadow in contact with a blob");

	defMultiValue("roi", "Region Of Interest, format: \"(x1 y1) (x2 y2) ...\"");
	defMultiValue("roni", "Region Of No Interest, format: \"(x1 y1) (x2 y2) ...\"");
}

void Bully::Params::set(const char *param, int value){
	set(param, static_cast<double>(value));
}
void Bully::Params::set(const char *param, float value){
	set(param, static_cast<double>(value));
}
void Bully::Params::set(const char *param, double value){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	Param &p = paramList[id];
	p.set(value);
}
void Bully::Params::set(const char *param, const char *value){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	Param &p = paramList[id];
	p.set(value);
}

double Bully::Params::getNumber(const char *param, int index){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	return paramList[id].getNumber(index);
}
string const & Bully::Params::getString(const char *param, int index){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	return paramList[id].getString(index);
}
string const & Bully::Params::getDesc(const char *param){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	return paramList[id].getDesc();
}
int Bully::Params::valueCount(const char * param){
	assert(param);
	string id(param);
	assert(paramList.count(id) > 0);
	return paramList[id].valueCount();
}

void Bully::Params::write(ostream &s){
	// Print the parameters in the order they were set.
	vector<Param*>::iterator it = paramSequence.begin();
	for( ; it != paramSequence.end(); it++ )
		(*it)->printValue(s);
}
void Bully::Params::write(const char * fileName){
	ofstream file(fileName);
	write(file);
}
void Bully::Params::print(){
	write(cout);
}
void Bully::Params::goToNextLine(ifstream &file){
	while( file.get() != '\n' && file.good() );
}
void Bully::Params::skipWhiteSpace(ifstream &file){
	while( file.good() && (file.peek() == ' ' || file.peek() == '\t') ) file.get();
}
void Bully::Params::read(const char * fileName){
	ifstream file(fileName);

	if( file.fail() )
		throw string("config file could not be opened: ") + string(fileName);

	for( int line = 1; file.good(); line++ ){

		// This is at the beginning of a line.
		// The format of the configuration file is extremely simple. A line must look like this:
		//   <parameter id> '=' <parameter value>
		//
		// ALL whitespace is ignored, along with lines starting with a #.

		// Check for comment.
		skipWhiteSpace(file);
		if( file.peek() == '#' ){
			goToNextLine(file);
			continue;
		}

		// Extract everything until '=' is encountered.
		stringstream ssid;
		for(;;){
			skipWhiteSpace(file);
			if( file.peek() == '=' || file.peek() == '\n' || !file.good()) break;
			ssid << (char)file.get();
		}

		string id;
		ssid >> id;

		// Check if this was the end of the file.
		if( id.length() == 0 ){
			if( file.peek() == '\n' ){
				file.get();
				continue; // Empty line, go to next.
			}
			if( file.eof() )
				break; // End of file.

			stringstream ss;
			ss << "an error occured while reading configuration file (line " << line << ")";
			throw ss.str();
		}

		// Match the parameter id to one already stored in paramList.
		if( paramList.count(id) <= 0 ){
			stringstream ss;
			ss << "unknown parameter id: " << id << " (line " << line << ")";
			throw ss.str();
		}

		if( file.get() != '=' || !file.good() ){
			stringstream ss;
			ss << "missing value for parameter: " << id << " (line " << line << ")";
			throw ss.str();
		}

		// Extract everything after the '='.
		stringstream ssval;
		for(;;){
			skipWhiteSpace(file);
			if( file.peek() == '\n' || file.peek() == '#' || !file.good()) break;
			ssval << (char)file.get();
		}

		Param &p = paramList[id];
		if( p.isNumber() ){
			double numval = 0.0;
			ssval >> numval;
			p.set(numval);
		}else if( p.isString() ){
			string strval;
			ssval >> strval;
			p.set(strval.c_str());
		}
		if( ssval.fail() ){
			stringstream ss;
			ss << "invalid parameter value (line " << line << ")";
			throw ss.str();
		}

		goToNextLine(file);
	}
}
void Bully::Params::printInfo(ostream &s){
	s	<< "Example configuration file \"bullyconfig.txt\":" << endl
		<< "---------------------------------------------" << endl
		<< "# Sample configuration file for Bully" << endl
		<< "inputFile = ./testvideo.avi   # a relative path is ok" << endl
		<< "modulePreprocess = hsv" << endl
		<< endl << endl
		<< "This has the effect of using the video \"testvideo.avi\" for input" << endl
		<< "and setting the preprocessing module to \"hsv\"." << endl << endl
		<< "Configuration variables not set explicitly get default values." << endl
		<< "A line *must* be in the format shown above, that is:" << endl
		<< "<name of variable> = <value>" << endl << endl
		<< "Note:" << endl
		<< "- Whitespace is ignored, so for instance empty lines are ok." << endl
		<< "- A line can begin with a # character, in which case the line is ignored." << endl
		<< "- A comment may also appear at the end of a line." << endl
		<< endl << endl
		<< "Some parameter can have multiple values. Each time such a parameter" << endl
		<< "is specified, a new value is appended to a list of values. Other" << endl
		<< "variables can have only one value at a time." << endl << endl
		<< "The available variables are:" << endl;

	vector<Param*>::iterator it = paramSequence.begin();
	for( ; it != paramSequence.end(); it++ ){
		s << (*it)->getId() << " (";
		(*it)->printType(s);
		s << ") : " << endl << "   ";

		string const & desc = (*it)->getDesc();
		size_t descpos = 0;
		size_t linepos = 3;
		for(;;){
			// Get one word.
			size_t space = desc.find_first_of(' ', descpos);
			size_t tab = desc.find_first_of('\t', descpos);
			size_t newline = desc.find_first_of('\n', descpos);
			size_t endpos = min(space, min(tab, newline));

			string word = desc.substr(descpos, endpos - descpos + 1);
			size_t wordsize = word.size();

			if( linepos + wordsize >= 80 ){
				s << endl << "   ";
				linepos = 3;
			}

			s << word;
			descpos = endpos + 1;
			linepos += wordsize;

			if( endpos < 0 || endpos >= desc.size() ){
				s << endl;
				break;
			}
		}
	}
}

void Bully::Params::getVector(std::string const &param, std::vector<Vector2i> &v){
	stringstream ss(param);
	char lpar, rpar;

	for(;;){
		Vector2i p;
		ss >> skipws >> lpar;
		if( lpar != '(' ) break;
		ss >> skipws >> p.x;
		ss >> skipws >> p.y;
		ss >> skipws >> rpar;
		if( rpar != ')' ) break;

		if( !ss.good() ) break;
		v.push_back(p);
	}
}

void Bully::Params::getVector(std::string const &param, std::vector<Vector2d> &v){
	stringstream ss(param);
	char lpar, rpar;

	for(;;){
		Vector2d p;
		ss >> skipws >> lpar;
		if( lpar != '(' ) break;
		ss >> skipws >> p.x;
		ss >> skipws >> p.y;
		ss >> skipws >> rpar;
		if( rpar != ')' ) break;

		if( !ss.good() ) break;
		v.push_back(p);
	}
}

void Bully::Params::getMatrix(std::string const &param, Matrix3<double> &m){
	stringstream ss(param);

	for(int r = 0; r < 3; r++){
		for(int c = 0; c < 3; c++){
			double val = 0.0;
			ss >> skipws >> val;
			m(r,c) = val;
		}
		if( !ss.good() ) break;
		char semicolon;
		ss >> skipws >> semicolon;
		if( semicolon != ';' ) break;
	}
}
