#pragma once

#include "bullytypes.h"
#include <ostream>
#include <string>
#include <vector>
#include <map>
#include <assert.h>

namespace Bully {

class Params {
public:
	Params();

	// Setting the value of a parameter.
	void					set(const char * param, int value);
	void					set(const char * param, float value);
	void					set(const char * param, double value);
	void					set(const char * param, const char *value);

	// Getting the value of a parameter.
	double					getNumber(const char * param, int index=0);
	std::string const &		getString(const char * param, int index=0);
	std::string const &		getDesc(const char * param);

	// Convenience for getting a number.
	double					get(const char * param, int index=0) { return getNumber(param,index); }

	// Querying the number of values currently stored in a MULTI_VALUE parameter
	int						valueCount(const char * param);

	void					print();
	void					printInfo(std::ostream &s);
	void					write(std::ostream &s);
	void					write(const char * fileName);
	void					read(const char * fileName);

	void getVector(std::string const &param, std::vector<Vector2i> &v);
	void getVector(std::string const &param, std::vector<Vector2d> &v);
	void getMatrix(std::string const &param, Matrix3<double> &m);

private:
	void defSingleValue(const char *id, int defaultValue, const char *desc);
	void defSingleValue(const char *id, double defaultValue, const char *desc);
	void defSingleValue(const char *id, const char *defaultValue, const char *desc);
	void defMultiValue(const char *id, int defaultValue, const char *desc);
	void defMultiValue(const char *id, double defaultValue, const char *desc);
	void defMultiValue(const char *id, const char *defaultValue, const char *desc);
	void defMultiValue(const char *id, const char *desc);
	void setDefaultValues();

	void skipWhiteSpace(std::ifstream &file);
	void goToNextLine(std::ifstream &file);

	class Param {
	public:
		enum TYPE { SINGLE_VALUE=0, MULTI_VALUE };

		Param() : _type(SINGLE_VALUE) {};

		std::string const & getId() const { return _id; }
		std::string const & getDesc() const { return _desc; }
		double				getNumber(int index) const { assert(_dvalue.size() > index); return _dvalue[index]; }
		std::string const & getString(int index) const { assert(_svalue.size() > index); return _svalue[index]; }

		TYPE type(){ return _type; }
		bool isNumber(){ return _dvalue.size() > 0; }
		bool isString(){ return _svalue.size() > 0; }
		bool defined(){ return isNumber() || isString(); }

		void def(const char *id, TYPE t, const char *desc);
		void set(double value);
		void set(const char *value);

		int valueCount();

		void printValue(std::ostream &s);
		void printType(std::ostream &s);

	private:
		TYPE _type;
		std::string _id;
		std::vector<double> _dvalue;
		std::vector<std::string> _svalue;
		std::string _desc;
	};

	std::map<std::string, Param> paramList;
	std::vector<Param*> paramSequence;
};

} // namespace Bully
