#pragma once

#include <vector>
#include "bullytypes.h"

namespace Bully {

class WorldObjectFrameInfo {
public:
	int frameId;				// The frame number this information is relevant for
	Vector2d imageFeetPos;		// The position of the feet in the 2D image
	Vector2d imageHeadPos;		// The position of the head in the 2D image
	Vector3d worldFeetPos;		// The position of the feet in the 2D floor plane (z component should be 1.0)
	double worldHeight;			// The height above the floor plane
};

class WorldObjectInfo {
public:
	std::vector<WorldObjectFrameInfo> frameInfo;

	// The complete world trajectory, including merges/splits.
	std::vector<Vector2d> completeWorldTrajectory;

	void calcStats( int frameCount );
	double meanspeed(){ return _meanspeed; }
	double minspeed(){ return _minspeed; }
	double maxspeed(){ return _maxspeed; }

private:
	double _meanspeed; // mean speed (meters per. frame)
	double _minspeed;
	double _maxspeed;
};

}
